﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;




namespace MP2LanguageConverter
{
    public partial class MP2LanguageConverterSetup : Form
    {
        private string SETTING_FILE = "MP2LanguageConverter.txt";
        private string LOG_FILE = "MP2LanguageConverter.log";
        System.Timers.Timer m_timer = null;  

        public MP2LanguageConverterSetup(string[] args)
        {
            InitializeComponent();
            textoutput("Mp2LanguageConverter");

            //arg processing for input file
            if (args.Length >= 1)
            {
                string argfile = args[0].Replace("\"","");
                if (File.Exists(argfile))
                {
                    SETTING_FILE = argfile;
                }
                
            }
            

            textoutput("Using log file " + LOG_FILE);
            if (File.Exists(LOG_FILE))
                File.Delete(LOG_FILE);
            
            if (File.Exists(SETTING_FILE))
            {
                textoutput("Using setting file " + SETTING_FILE);

                string filestring = File.ReadAllText(SETTING_FILE);
                filestring = filestring.Replace("\r", string.Empty);
                string[] tokenarray = filestring.Split('\n');

                if (tokenarray.Length > 0)
                {
                    textBoxInputFolder.Text = tokenarray[0].Replace("LANGUAGE_INPUT_FOLDER=", String.Empty);
                }
                if (tokenarray.Length > 1)
                {
                    textBoxOutputFolder.Text = tokenarray[1].Replace("LANGUAGE_OUTPUT_FOLDER=", String.Empty);
                }
                if (tokenarray.Length > 2)
                {
                    textBoxPluginName.Text = tokenarray[2].Replace("PLUGIN_NAME=", String.Empty);
                }
                if (tokenarray.Length > 3)
                {
                    numericUpDownTabReplace.Value = Convert.ToInt32(tokenarray[3].Replace("REPLACE_TABS=", String.Empty));
                }
                if (tokenarray.Length > 4)
                {
                    radioButtonMP2Xmas.Checked = Convert.ToBoolean(tokenarray[4].Replace("MP2XMAS_FORMAT=", String.Empty));
                }
                if (tokenarray.Length > 5)
                {
                    radioButtonMP2Summer.Checked = Convert.ToBoolean(tokenarray[5].Replace("MP2_FORMAT=", String.Empty));
                }
                if (tokenarray.Length > 6)
                {
                    radioButtonM1Format.Checked = Convert.ToBoolean(tokenarray[6].Replace("MP1_FORMAT=", String.Empty));
                }
            }

            //arg processing for start processing
            if (args.Length >= 2)
            {
                if (args[1].ToUpper()=="PROCESS")
                {
                    m_timer = new System.Timers.Timer(10); //close after 0.1s
                    m_timer.Enabled = true;
                    m_timer.Elapsed += new System.Timers.ElapsedEventHandler(autoprocessing);
                }
            }

            
            
        }

        public void autoprocessing(object sender, System.Timers.ElapsedEventArgs e)
        {
            if (m_timer != null)
                m_timer.Elapsed -= new System.Timers.ElapsedEventHandler(autoprocessing);

            ProcessFolder(textBoxInputFolder.Text, textBoxOutputFolder.Text);
            System.Threading.Thread.Sleep(2000);
            Application.Exit();         
        }

        private void buttonInputFolder_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog folderDialog = new FolderBrowserDialog();
            folderDialog.Description = "InputFolder: Select MP1 Language Folder (Contains strings_en.xml)";
            folderDialog.ShowNewFolderButton = false;
            if (folderDialog.ShowDialog() == DialogResult.OK)
            {
                textoutput("User selected folder is " + folderDialog.SelectedPath);
                textBoxInputFolder.Text = folderDialog.SelectedPath;
            }
            else
            {
                textoutput("User aborted" + "\n");
            }
        }

        private void buttonOutputFolder_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog folderDialog = new FolderBrowserDialog();
            folderDialog.Description = "OutputFolder: Select MP2 Language Folder)";
            folderDialog.ShowNewFolderButton = false;
            if (folderDialog.ShowDialog() == DialogResult.OK)
            {
                textoutput("User selected folder is " + folderDialog.SelectedPath);
                textBoxOutputFolder.Text = folderDialog.SelectedPath;
            }
            else
            {
                textoutput("User aborted" + "\n");
            }
        }

        private void buttonExit_Click(object sender, EventArgs e)
        {
            File.WriteAllText(SETTING_FILE, "LANGUAGE_INPUT_FOLDER=" + textBoxInputFolder.Text + "\r\n" + "LANGUAGE_OUTPUT_FOLDER=" + textBoxOutputFolder.Text + "\r\n" + "PLUGIN_NAME=" + textBoxPluginName.Text + "\r\n" + "REPLACE_TABS=" + numericUpDownTabReplace.Value.ToString() + "\r\n" + "MP2XMAS_FORMAT=" + radioButtonMP2Xmas.Checked.ToString() + "\r\n" + "MP2_FORMAT=" + radioButtonMP2Summer.Checked.ToString() + "\r\n" + "MP1_FORMAT=" + radioButtonM1Format.Checked.ToString());
            Close();
        }

        private void buttonProcess_Click(object sender, EventArgs e)
        {
            ProcessFolder(textBoxInputFolder.Text, textBoxOutputFolder.Text);
        }


        public void ProcessFolder(string inputfoldername, string outputfoldername)
        {
            try
            {
                DirectoryInfo dirinfo = new DirectoryInfo(inputfoldername);
                FileInfo[] allfiles = dirinfo.GetFiles("*.xml");

                foreach (FileInfo myfileinfo in allfiles)
                {
                    string inputfilename = myfileinfo.FullName;
                    //textoutput("inputfilename=" + inputfilename);
                    string outputfilename = outputfoldername + @"\" + myfileinfo.Name;
                    //textoutput("outputfilename=" + outputfilename);

                    if (radioButtonMP2Xmas.Checked)
                        ProcessFile1(inputfilename, outputfilename);
                    else if (radioButtonMP2Summer.Checked)
                        ProcessFile2(inputfilename, outputfilename);
                    else if (radioButtonM1Format.Checked)
                        ProcessFile3(inputfilename, outputfilename);
                }
                textoutput("");
            }
            catch (Exception exc)
            {
                textoutput("ProcessFolder Exception: " + exc.Message);
            }
        }


        public void ProcessFile1(string inputfilename, string outputfilename)
        {
            textoutput("ProcessFile=" + inputfilename);
            try
            {
                //read inputfile
                string[] inputlines = File.ReadAllLines(inputfilename);
                //textoutput("  " + inputlines.Length.ToString()+" lines read");

                string outputtext = "";
                string MP2String = "";
                bool useMP2String = false;
                string outputline = "";

                int string_ctr = 0;
                
                
                //process each line
                foreach (string line in inputlines)
                {

                    //textoutput("line=" + line);                   
                    string spaces = string.Empty;
                    for (int j = 0; j < numericUpDownTabReplace.Value; j++)
                    {
                        spaces+=" ";
                    }
                    string myline = line.Replace("\t", spaces); //4 spaces for one tab

                    //global replacements
                    if (myline.Contains("<Language")) 
                    {
                        myline = "<Language>";
                        MP2String = "";
                        useMP2String = false;
                    }
                    else if (myline.Contains("</Language")) 
                    {
                        myline = "</Language>";
                        MP2String = "";
                        useMP2String = false;
                    }
                    else if (myline.Contains("<Section name"))
                    {
                        myline = "    <Section Name=\""+textBoxPluginName.Text+"\">";
                        MP2String = "";
                        useMP2String = false;
                    }
                    else if (myline.Contains("</Section>"))
                    {
                        myline = "    </Section>";
                        MP2String = "";
                        useMP2String = false;
                    }
                    //end global replacements
                    
                    
                    string templine = myline;




                    /*
                    for (int i = 0; i < templine.Length; i++)
                    {
                        int k = (int)templine[i];
                        textoutput("zeichen=" + templine[i].ToString() + "   char=" + k.ToString());
                    }*/

                    string leadingSpaceString = "";

                    //remove leading spaces
                    int leadingSpaces=0;
                    for (int i = 0; i < templine.Length; i++)
                    {
                        if (templine[i] == ' ')
                        {
                            leadingSpaces++;
                            leadingSpaceString += " ";
                        }
                        else
                        {
                            break;
                        }
                    }
                    //textoutput("leadingSpaces=" + leadingSpaces.ToString());
                    templine = templine.Substring(leadingSpaces, templine.Length - leadingSpaces);//remove leading spaces

                    //remove trailing spaces
                    int trailingSpaces = 0;
                    for (int i = templine.Length - 1; i >= 0; i--)
                    {
                        if (templine[i] == ' ')
                        {
                            trailingSpaces++;
                        }
                        else
                        {
                            break;
                        }
                    }
                    //textoutput("trailingSpaces=" + trailingSpaces.ToString());
                    templine = templine.Substring(0, templine.Length - trailingSpaces);

                    //textoutput("nospaces: templine=" + templine);
                    //textoutput("count=" + templine.Length.ToString());

                    //no process
                    if (templine.Contains("<!-- MP2KEY:"))  // <!--MP2KEY name -->
                    {
                        templine = templine.Substring(12, templine.Length - 12);  //remove beginning comment and keyword                       
                        templine = templine.Replace(" -->", "");// remove ending comment
                        templine = templine.Replace("-->", "");// remove ending comment

                        //now you have the MP2 line
                        outputline = leadingSpaceString + templine;
                        MP2String = "";
                        useMP2String = false;
                    }
                    if (templine.Contains("<!--MP2KEY:"))  // <!--MP2KEY name -->
                    {
                        templine = templine.Substring(11, templine.Length - 11);  //remove beginning comment and keyword                       
                        templine = templine.Replace(" -->", "");// remove ending comment
                        templine = templine.Replace("-->", "");// remove ending comment

                        //now you have the MP2 line
                        outputline = leadingSpaceString + templine;
                        MP2String = "";
                        useMP2String = false;
                    }
                    else if (templine.Contains("<!-- MP2STR:"))  // <!-- MP2STR name -->
                    {
                        templine = templine.Substring(12, templine.Length - 12);  //remove beginning comment and keyword  
                        templine = templine.Replace(" -->", "");// remove ending comment
                        templine = templine.Replace("-->", "");// remove ending comment

                        //now you have the MP2 string name
                        MP2String = templine;
                        useMP2String = true;
                        outputline = "--skipline--";
                    }
                    else if (templine.Contains("<!--MP2STR:"))  // <!--MP2STR name -->
                    {
                        templine = templine.Substring(11, templine.Length - 11);  //remove beginning comment and keyword  
                        templine = templine.Replace(" -->", "");// remove ending comment
                        templine = templine.Replace("-->", "");// remove ending comment
                        //now you have the MP2 string name
                        MP2String = templine;
                        useMP2String = true;
                        outputline = "--skipline--";
                    }
                    else if (templine.Contains("<!--")) //comment is copied to MP2 without changes
                    {
                        outputline = myline;
                        MP2String = "";
                        useMP2String = false;
                    }
                    else if (templine.Contains("<String id=\""))  // <String id="1107">Mehr</String> 
                    {
                        templine = templine.Substring(12, templine.Length - 12);  //remove beginning comment and <String id=" 

                        //textoutput("templine=" + templine);

                        //now extract the numbers till the next "
                        int digits = 0;
                        for (int i = 0; i < templine.Length; i++)
                        {
                            if (templine[i] == '"')
                            {
                                break;
                            }
                            else
                            {
                                digits++;
                            }
                        }
                        //textoutput("digits=" + digits.ToString());

                        string digitString = templine.Substring(0, digits);
                        if (templine.Length >= digits)
                            templine = templine.Substring(digits, templine.Length - digits);

                        templine = templine.Replace("\">", ""); //remove ending

                        //textoutput("templine=" + templine);
                       
                        templine = templine.Replace("</String>",""); //remove ending 

                        //textoutput("templine=" + templine);

                        string valueString = templine;

                        valueString = SpecialProcessing(valueString, digitString);

                        // <String Name="MenuEntry" Text="MyTestPlugin"/>

                        if (useMP2String)
                        {
                            outputline = leadingSpaceString + "<!-- MP2STR:" + digitString + " -->" + Environment.NewLine + leadingSpaceString + "<String Name=\"" + MP2String + "\" Text=\"" + valueString + "\"/>";
                            MP2String = "";
                            useMP2String = false;
                        }
                        else
                        {
                            outputline = leadingSpaceString + "<String Name=\"" + digitString + "\" Text=\"" + valueString + "\"/>";
                        }

                        string_ctr++;
                    }
                    else //transfer without changes for MP2 e.g. // <!--  Buttons -->
                    {
                        outputline = myline;
                        MP2String = "";
                        useMP2String = false;
                    }

                    if ((!useMP2String) && (outputline != "--skipline--"))
                    {
                        //textoutput("outputline=" + outputline);
                        outputtext += outputline + Environment.NewLine;
                    }
                }

                //write outputfile
                if (File.Exists(outputfilename))
                {
                    if (File.Exists(outputfilename + ".bak"))
                        File.Delete(outputfilename + ".bak");

                    File.Move(outputfilename,outputfilename+".bak");
                }
                File.WriteAllText(outputfilename, outputtext);

                textoutput(string_ctr.ToString()+" Strings written to file");
                textoutput(outputfilename+"\n\n\n");

            }
            catch (Exception exc)
            {
                textoutput("ProcessFile Exception: " + exc.Message);
            }
        }


        public void ProcessFile2(string inputfilename, string outputfilename)
        {
            textoutput("ProcessFile=" + inputfilename);
            try
            {
                //read inputfile
                string[] inputlines = File.ReadAllLines(inputfilename);
                //textoutput("  " + inputlines.Length.ToString()+" lines read"); 

                string outputtext = "";
                string MP2String = "";
                bool useMP2String = false;
                string outputline = "";

                int string_ctr = 0;


                //process each line
                foreach (string line in inputlines)
                {

                    //textoutput("line=" + line);                   
                    string spaces = string.Empty;
                    for (int j = 0; j < numericUpDownTabReplace.Value; j++)
                    {
                        spaces += " ";
                    }
                    string myline = line.Replace("\t", spaces); //4 spaces for one tab

                    //global replacements
                    if (myline.Contains("<Language"))
                    {
                        myline = "<resources>";
                        MP2String = "";
                        useMP2String = false;
                    }
                    else if (myline.Contains("</Language"))
                    {
                        myline = "</resources>";
                        MP2String = "";
                        useMP2String = false;
                    }
                    else if (myline.Contains("<Section name"))
                    {
                        MP2String = "";
                        useMP2String = false;
                        continue;
                    }
                    else if (myline.Contains("</Section>"))
                    {
                        MP2String = "";
                        useMP2String = false;
                        continue;
                    }
                    //end global replacements


                    string templine = myline;




                    /*
                    for (int i = 0; i < templine.Length; i++)
                    {
                        int k = (int)templine[i];
                        textoutput("zeichen=" + templine[i].ToString() + "   char=" + k.ToString());
                    }*/

                    string leadingSpaceString = "";

                    //remove leading spaces
                    int leadingSpaces = 0;
                    for (int i = 0; i < templine.Length; i++)
                    {
                        if (templine[i] == ' ')
                        {
                            leadingSpaces++;
                            leadingSpaceString += " ";
                        }
                        else
                        {
                            break;
                        }
                    }
                    //textoutput("leadingSpaces=" + leadingSpaces.ToString());
                    templine = templine.Substring(leadingSpaces, templine.Length - leadingSpaces);//remove leading spaces

                    //remove trailing spaces
                    int trailingSpaces = 0;
                    for (int i = templine.Length - 1; i >= 0; i--)
                    {
                        if (templine[i] == ' ')
                        {
                            trailingSpaces++;
                        }
                        else
                        {
                            break;
                        }
                    }
                    //textoutput("trailingSpaces=" + trailingSpaces.ToString());
                    templine = templine.Substring(0, templine.Length - trailingSpaces);

                    //textoutput("nospaces: templine=" + templine);
                    //textoutput("count=" + templine.Length.ToString());


                    if (templine.Contains("<!-- MP2STR:"))  // <!-- MP2STR name -->
                    {
                        templine = templine.Substring(12, templine.Length - 12);  //remove beginning comment and keyword  
                        templine = templine.Replace(" -->", "");// remove ending comment
                        templine = templine.Replace("-->", "");// remove ending comment

                        //now you have the MP2 string name
                        MP2String = templine;
                        useMP2String = true;
                        outputline = "--skipline--";
                    }
                    else if (templine.Contains("<!--MP2STR:"))  // <!--MP2STR name -->
                    {
                        templine = templine.Substring(11, templine.Length - 11);  //remove beginning comment and keyword  
                        templine = templine.Replace(" -->", "");// remove ending comment
                        templine = templine.Replace("-->", "");// remove ending comment
                        //now you have the MP2 string name
                        MP2String = templine;
                        useMP2String = true;
                        outputline = "--skipline--";
                    }
                    else if (templine.Contains("<!--")) //comment is copied to MP2 without changes
                    {
                        outputline = myline;
                        MP2String = "";
                        useMP2String = false;
                    }
                    else if (templine.Contains("<String id=\""))  // <String id="1107">Mehr</String> 
                    {
                        templine = templine.Substring(12, templine.Length - 12);  //remove beginning comment and <String id=" 

                        //textoutput("templine=" + templine);

                        //now extract the numbers till the next "
                        int digits = 0;
                        for (int i = 0; i < templine.Length; i++)
                        {
                            if (templine[i] == '"')
                            {
                                break;
                            }
                            else
                            {
                                digits++;
                            }
                        }
                        //textoutput("digits=" + digits.ToString());

                        string digitString = templine.Substring(0, digits);
                        if (templine.Length >= digits)
                            templine = templine.Substring(digits, templine.Length - digits);

                        templine = templine.Replace("\">", ""); //remove ending

                        //textoutput("templine=" + templine);

                        templine = templine.Replace("</String>", ""); //remove ending 

                        //textoutput("templine=" + templine);

                        string valueString = templine;

                        valueString = SpecialProcessing(valueString, digitString);

                        // <string name="Weather.Humidity">Humidity</string>

                        if (useMP2String)
                        {
                            outputline = leadingSpaceString + "<!-- MP1STR:" + digitString + " -->" + Environment.NewLine + leadingSpaceString + "<string name=\"" + textBoxPluginName.Text + "." + MP2String + "\">" + valueString + "</string>";
                            MP2String = "";
                            useMP2String = false;
                        }
                        else
                        {
                            outputline = leadingSpaceString + "<string name=\"" + textBoxPluginName.Text + "." + digitString + "\">" + valueString + "</string>";
                        }

                        string_ctr++;
                    }
                    else //transfer without changes for MP2 e.g. // <!--  Buttons -->
                    {
                        outputline = myline;
                        MP2String = "";
                        useMP2String = false;
                    }

                    if ((!useMP2String) && (outputline != "--skipline--"))
                    {
                        //textoutput("outputline=" + outputline);
                        outputtext += outputline + Environment.NewLine;
                    }
                }

                //write outputfile
                if (File.Exists(outputfilename))
                {
                    if (File.Exists(outputfilename + ".bak"))
                        File.Delete(outputfilename + ".bak");

                    File.Move(outputfilename, outputfilename + ".bak");
                }
                File.WriteAllText(outputfilename, outputtext);

                textoutput(string_ctr.ToString() + " Strings written to file");
                textoutput(outputfilename + "\n\n\n");

            }
            catch (Exception exc)
            {
                textoutput("ProcessFile Exception: " + exc.Message);
            }
        }


        public void ProcessFile3(string inputfilename, string outputfilename)
        {
            textoutput("ProcessFile=" + inputfilename);
            try
            {
                //read inputfile
                string[] inputlines = File.ReadAllLines(inputfilename);
                //textoutput("  " + inputlines.Length.ToString()+" lines read");

                string outputtext = "";
                string MP2String = "";
                bool useMP2String = false;
                string outputline = "";

                int string_ctr = 0;

                textoutput("Converting transifexreturnline");
                Char transifexreturnline_char = Convert.ToChar(9166);
                string transifexreturnline_string = transifexreturnline_char.ToString();
                textoutput("Converting transifexreturnline=" + transifexreturnline_string);
                //process each line
                foreach (string line in inputlines)
                {

                    //textoutput("line=" + line);                   
                    string spaces = string.Empty;
                    for (int j = 0; j < numericUpDownTabReplace.Value; j++)
                    {
                        spaces += " ";
                    }
                    string myline = line.Replace("\t", spaces); //4 spaces for one tab

                    myline = myline.Replace("\r",string.Empty);
                    myline = myline.Replace("\n", @"\n");
                    myline = myline.Replace(@"\'", "'");

                    myline = myline.Replace(transifexreturnline_string, @"\n");



                    //global replacements
                    if (myline.Contains("<resources>"))
                    {
                        myline = "<Language>"+ Environment.NewLine + "  <Section name=\"unmapped\">";
                        MP2String = "";
                        useMP2String = false;
                    }
                    else if (myline.Contains("</resources>"))
                    {
                        myline = "  </Section>" + Environment.NewLine + "</Language>";
                        MP2String = "";
                        useMP2String = false;
                    }
                    
                    //end global replacements


                    string templine = myline;




                    /*
                    for (int i = 0; i < templine.Length; i++)
                    {
                        int k = (int)templine[i];
                        textoutput("zeichen=" + templine[i].ToString() + "   char=" + k.ToString());
                    }*/

                    string leadingSpaceString = "";

                    //remove leading spaces
                    int leadingSpaces = 0;
                    for (int i = 0; i < templine.Length; i++)
                    {
                        if (templine[i] == ' ')
                        {
                            leadingSpaces++;
                            leadingSpaceString += " ";
                        }
                        else
                        {
                            break;
                        }
                    }
                    //textoutput("leadingSpaces=" + leadingSpaces.ToString());
                    templine = templine.Substring(leadingSpaces, templine.Length - leadingSpaces);//remove leading spaces

                    //remove trailing spaces
                    int trailingSpaces = 0;
                    for (int i = templine.Length - 1; i >= 0; i--)
                    {
                        if (templine[i] == ' ')
                        {
                            trailingSpaces++;
                        }
                        else
                        {
                            break;
                        }
                    }
                    //textoutput("trailingSpaces=" + trailingSpaces.ToString());
                    templine = templine.Substring(0, templine.Length - trailingSpaces);

                    //textoutput("nospaces: templine=" + templine);
                    //textoutput("count=" + templine.Length.ToString());

                    if (templine.Contains("<!-- MP1STR:"))  // <!-- MP2STR name -->
                    {
                        templine = templine.Substring(12, templine.Length - 12);  //remove beginning comment and keyword  
                        templine = templine.Replace(" -->", "");// remove ending comment
                        templine = templine.Replace("-->", "");// remove ending comment

                        //now you have the MP2 string name
                        MP2String = templine;
                        useMP2String = true;
                        outputline = "--skipline--";
                    }
                    else if (templine.Contains("<!--MP1STR:"))  // <!--MP2STR name -->
                    {
                        templine = templine.Substring(11, templine.Length - 11);  //remove beginning comment and keyword  
                        templine = templine.Replace(" -->", "");// remove ending comment
                        templine = templine.Replace("-->", "");// remove ending comment
                        //now you have the MP2 string name
                        MP2String = templine;
                        useMP2String = true;
                        outputline = "--skipline--";
                    }
                    else if (templine.Contains("<!--")) //comment is copied to MP2 without changes
                    {
                        outputline = myline;
                        MP2String = "";
                        useMP2String = false;
                    }
                    else if (templine.Contains("<string name=\""))  // <string name="TvWishListMP2.100">TVWishList</string> 
                    {
                        templine = templine.Substring(14, templine.Length - 14);  //remove beginning comment and <string name=" 

                        //textoutput("templine=" + templine);

                        //now extract the string till the next "
                        int digits = 0;
                        for (int i = 0; i < templine.Length; i++)
                        {
                            if (templine[i] == '"')
                            {
                                break;
                            }
                            else
                            {
                                digits++;
                            }
                        }
                        //textoutput("digits=" + digits.ToString());

                        string digitString = templine.Substring(0, digits);
                        digitString = digitString.Replace(textBoxPluginName.Text+".",string.Empty);

                        if (templine.Length >= digits)
                            templine = templine.Substring(digits, templine.Length - digits);

                        templine = templine.Replace("\">", ""); //remove ending

                        //textoutput("templine=" + templine);

                        templine = templine.Replace("</string>", ""); //remove ending 

                        //textoutput("templine=" + templine);

                        string valueString = templine;

                        valueString = SpecialProcessing(valueString, digitString);

                        // <string name="Weather.Humidity">Humidity</string>

                        if (useMP2String)   //<String id="1104">New</String>
                        {
                            outputline = leadingSpaceString + "<!-- MP2STR:" + digitString + " -->" + Environment.NewLine + leadingSpaceString + "<String id=\"" + MP2String + "\">" + valueString + "</String>";
                            MP2String = "";
                            useMP2String = false;
                        }
                        else
                        {
                            outputline = leadingSpaceString + "<String id=\"" + digitString + "\">" + valueString + "</String>";
                        }

                        string_ctr++;
                    }
                    else //transfer without changes for MP2 e.g. // <!--  Buttons -->
                    {
                        outputline = myline;
                        MP2String = "";
                        useMP2String = false;
                    }

                    if ((!useMP2String) && (outputline != "--skipline--"))
                    {
                        //textoutput("outputline=" + outputline);
                        outputtext += outputline + Environment.NewLine;
                    }
                }

                //write outputfile
                if (File.Exists(outputfilename))
                {
                    if (File.Exists(outputfilename + ".bak"))
                        File.Delete(outputfilename + ".bak");

                    File.Move(outputfilename, outputfilename + ".bak");
                }
                File.WriteAllText(outputfilename, outputtext);

                textoutput(string_ctr.ToString() + " Strings written to file");
                textoutput(outputfilename + "\n\n\n");

            }
            catch (Exception exc)
            {
                textoutput("ProcessFile Exception: " + exc.Message);
            }
        }

        
        private void textoutput(string textlines)
        {
            try
            {
                File.AppendAllText(LOG_FILE, textlines+"\r\n");
                string text = "";

                if (listView1 != null)
                {
                    Color mycolor;
                    if (textlines.StartsWith("<RED>") == true)
                    {//color red
                        textlines = textlines.Substring(5, textlines.Length - 5);
                        mycolor = Color.Red;
                    }
                    else if (textlines.StartsWith("<YELLOW>") == true)
                    {//color yellow
                        textlines = textlines.Substring(8, textlines.Length - 8);
                        mycolor = Color.Orange;
                    }
                    else if (textlines.StartsWith("<GREEN>") == true)
                    {//color green
                        textlines = textlines.Substring(8, textlines.Length - 8);
                        mycolor = Color.Green;
                    }
                    else
                    {//color black
                        mycolor = Color.Black;
                    }

                    char[] splitterchars = { '\n' };  //split lines with \n
                    string[] lines = textlines.Split(splitterchars);
                    foreach (string line in lines)
                    {
                        /*try
                        {
                            Status.WriteLine(color_string + line);
                        }
                        catch //do nothing if stream writer is not open
                        {
                        }*/


                        text = line;
                        while (text.Length > 70)   //split long lines
                        {
                            int linelength = 69;
                            for (int i = 69; i >= 4; i--)
                            {
                                if (text[i] == ' ')
                                {
                                    linelength = i;
                                    break;
                                }
                            }
                            string pretext = text.Substring(0, linelength);

                            listView1.Items.Add(pretext);
                            listView1.Items[listView1.Items.Count - 1].ForeColor = mycolor;
                            listView1.Items[listView1.Items.Count - 1].EnsureVisible();



                            text = "+  " + text.Substring(linelength, text.Length - linelength);

                        }

                        listView1.Items.Add(text);
                        listView1.Items[listView1.Items.Count - 1].ForeColor = mycolor;
                        listView1.Items[listView1.Items.Count - 1].EnsureVisible();

                    }

                }
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message,"Error:");
            }
        }

        private string SpecialProcessing(string text, string tag)
        {
            /*
            if (text.Length > 40)
            {
                //textoutput("Long Text: tag=" + tag + "   text=" + text + "   length=" + text.Length.ToString());
                //textoutput("");
            }

            if (text.Contains("\\n"))
            {
                //textoutput("LineReturn: tag=" + tag + "   text=" + text + "   contains line break");
                //textoutput("");
            }*/

            /*if (tag == "1900")
            {
                textoutput(" debug line break:");
                int ctr = 0;
                foreach (char mychar in text)
                {
                    textoutput(ctr.ToString() + "   " + mychar.ToString() + "   " + ((int)mychar).ToString());
                    ctr++;
                }
                textoutput("");
                textoutput("");
                textoutput("");
            }*/

            


            return text;
        }

        
        
        
    }
}
